@@ -26,12 +26,13 @@
 #include <regex.h>
 #include <string.h>
 #include <dirent.h>
+#include "swupdate_dict.h"
+#include "swupdate_image.h"
 
 #if defined(__linux__)
 #include <sys/statvfs.h>
 #endif
 
-#include "swupdate.h"
 #include "util.h"
 #include "generated/autoconf.h"
 
@@ -322,71 +323,6 @@ int mkpath(char *dir, mode_t mode)
 	return 0;
 }
 
-/*
- * This function is strict bounded with the hardware
- * It reads some GPIOs to get the hardware revision
- */
-int get_hw_revision(struct hw_type *hw)
-{
-	FILE *fp;
-	int ret;
-	char *b1, *b2;
-#ifdef CONFIG_HW_COMPATIBILITY_FILE
-#define HW_FILE CONFIG_HW_COMPATIBILITY_FILE
-#else
-#define HW_FILE "/etc/hwrevision"
-#endif
-
-	if (!hw)
-		return -EINVAL;
-
-	/*
-	 * do not overwrite if it is already set
-	 * (maybe from command line)
-	 */
-	if (strlen(hw->boardname))
-		return 0;
-
-	memset(hw->boardname, 0, sizeof(hw->boardname));
-	memset(hw->revision, 0, sizeof(hw->revision));
-
-	/*
-	 * Not all boards have pins for revision number
-	 * check if there is a file containing theHW revision number
-	 */
-	fp = fopen(HW_FILE, "r");
-	if (!fp)
-		return -1;
-
-	ret = fscanf(fp, "%ms %ms", &b1, &b2);
-	fclose(fp);
-
-	if (ret != 2) {
-		TRACE("Cannot find Board Revision");
-		if(ret == 1)
-			free(b1);
-		return -1;
-	}
-
-	if ((strlen(b1) > (SWUPDATE_GENERAL_STRING_SIZE) - 1) ||
-		(strlen(b2) > (SWUPDATE_GENERAL_STRING_SIZE - 1))) {
-		ERROR("Board name or revision too long");
-		ret = -1;
-		goto out;
-	}
-
-	strlcpy(hw->boardname, b1, sizeof(hw->boardname));
-	strlcpy(hw->revision, b2, sizeof(hw->revision));
-
-	ret = 0;
-
-out:
-	free(b1);
-	free(b2);
-
-	return ret;
-}
-
 /**
  * hwid_match - try to match a literal or RE hwid
  * @rev: literal or RE specification
@@ -428,39 +364,6 @@ out:
 	return ret;
 }
 
-/*
- * The HW revision of the board *MUST* be inserted
- * in the sw-description file
- */
-#ifdef CONFIG_HW_COMPATIBILITY
-int check_hw_compatibility(struct swupdate_cfg *cfg)
-{
-	struct hw_type *hw;
-	int ret;
-
-	ret = get_hw_revision(&cfg->hw);
-	if (ret < 0)
-		return -1;
-
-	TRACE("Hardware %s Revision: %s", cfg->hw.boardname, cfg->hw.revision);
-	LIST_FOREACH(hw, &cfg->hardware, next) {
-		if (hw &&
-		    (!hwid_match(hw->revision, cfg->hw.revision))) {
-			TRACE("Hardware compatibility verified");
-			return 0;
-		}
-	}
-
-	return -1;
-}
-#else
-int check_hw_compatibility(struct swupdate_cfg
-		__attribute__ ((__unused__)) *cfg)
-{
-	return 0;
-}
-#endif
-
 uintmax_t
 from_ascii (char const *where, size_t digs, unsigned logbase)
 {
