@@ -20,7 +20,7 @@
 #include <archive.h>
 #include <archive_entry.h>
 
-#include "swupdate.h"
+#include "swupdate_image.h"
 #include "handler.h"
 #include "util.h"
 
