@@ -12,7 +12,7 @@
 #include <stdlib.h>
 #include <errno.h>
 
-#include "swupdate.h"
+#include "swupdate_image.h"
 #include "handler.h"
 #include "util.h"
 
