@@ -25,8 +25,8 @@
 #include <sys/ioctl.h>
 
 #include <mtd/mtd-user.h>
-#include "swupdate.h"
 #include "handler.h"
+#include "swupdate_image.h"
 #include "util.h"
 #include "flash.h"
 #include "progress.h"
@@ -308,9 +308,14 @@ static int flash_write_nor(int mtdnum, struct img_type *img)
 		return -ENODEV;
 	}
 
-	if(flash_erase_sector(mtdnum, img->seek, img->size)) {
-		ERROR("I cannot erasing %s",
-			img->device);
+	long long size = get_output_size(img, true);
+	if (size < 0) {
+		ERROR("Failed to determine output size, bailing out.");
+		return -1;
+	}
+	if (flash_erase_sector(mtdnum, img->seek, size)) {
+		ERROR("Failed to erase sectors on /dev/mtd%d (start: %llu, size: %lld)",
+			mtdnum, img->seek, size);
 		return -1;
 	}
 
