@@ -24,7 +24,7 @@
 #include <errno.h>
 #include <libgen.h>
 
-#include "swupdate.h"
+#include "swupdate_image.h"
 #include "handler.h"
 #include "util.h"
 
