@@ -34,6 +34,7 @@
 #include <swupdate_settings.h>
 #include <swupdate_dict.h>
 #include "suricatta_private.h"
+#include "suricatta/server.h"
 
 #define CONFIG_SECTION "suricatta"
 
@@ -77,19 +78,6 @@ static channel_data_t channel_data_defaults = {
 	.source = SOURCE_SURICATTA,
 };
 
-/*
- * Prototypes for "public" functions implementing the server
- * interface specified in include/suricatta/server.h.
- */
-void server_print_help(void);
-unsigned int server_get_polling_interval(void);
-server_op_res_t server_has_pending_action(int *action_id);
-server_op_res_t server_start(char *fname, int argc, char *argv[]);
-server_op_res_t server_stop(void);
-server_op_res_t server_install_update(void);
-server_op_res_t server_ipc(ipc_message *msg);
-server_op_res_t server_send_target_data(void);
-
 /* Global Lua state for this Suricatta Lua module implementation. */
 static lua_State *gL = NULL;
 
@@ -179,23 +167,23 @@ typedef struct {
 	 __builtin_choose_expr(__builtin_types_compatible_p( \
 		typeof(name), char*          ), lua_pushstring, \
 	 __builtin_choose_expr(__builtin_types_compatible_p( \
-		typeof(name), int            ), lua_pushnumber, \
+		typeof(name), int            ), lua_pushinteger, \
 	 (void)0)))(L, name) \
 	); \
 	(__builtin_choose_expr(__builtin_types_compatible_p( \
 		typeof(value), bool          ), lua_pushboolean, \
 	 __builtin_choose_expr(__builtin_types_compatible_p( \
-		typeof(value), unsigned int  ), lua_pushnumber, \
+		typeof(value), unsigned int  ), lua_pushinteger, \
 	 __builtin_choose_expr(__builtin_types_compatible_p( \
-		typeof(value), int           ), lua_pushnumber, \
+		typeof(value), int           ), lua_pushinteger, \
 	 __builtin_choose_expr(__builtin_types_compatible_p( \
-		typeof(value), long long     ), lua_pushnumber, \
+		typeof(value), long long     ), lua_pushinteger, \
 	 __builtin_choose_expr(__builtin_types_compatible_p( \
-		typeof(value), long          ), lua_pushnumber, \
+		typeof(value), long          ), lua_pushinteger, \
 	 __builtin_choose_expr(__builtin_types_compatible_p( \
 		typeof(value), double        ), lua_pushnumber, \
 	 __builtin_choose_expr(__builtin_types_compatible_p( \
-		typeof(value), channel_body_t), lua_pushnumber, \
+		typeof(value), channel_body_t), lua_pushinteger, \
 	 __builtin_choose_expr(__builtin_types_compatible_p( \
 		typeof(value), char[]         ), lua_pushstring, \
 	 __builtin_choose_expr(__builtin_types_compatible_p( \
@@ -311,9 +299,11 @@ static int json_push_to_table(lua_State *L, json_object *jsobj)
 		lua_pushboolean(L, json_object_get_boolean(jsobj));
 		break;
 	case json_type_double:
-	case json_type_int:
 		lua_pushnumber(L, json_object_get_int(jsobj));
 		break;
+	case json_type_int:
+		lua_pushinteger(L, json_object_get_int(jsobj));
+		break;
 	case json_type_null:
 		/* Lua has no notion of 'null', mimic it by an empty Table. */
 		lua_createtable(L, 0, 0);
@@ -360,7 +350,7 @@ static int json_to_table_callback(json_object *jsobj, int flags,
 	}
 	if (jsobj_index && !jsobj_key) {
 		/* Visiting array element: push to Lua "Array" Table part. */
-		lua_pushnumber(L, (int)*jsobj_index + 1);
+		lua_pushinteger(L, (int)*jsobj_index + 1);
 	} else {
 		/* Visiting object element: push to Lua "Dict" Table part. */
 		lua_pushstring(L, jsobj_key);
@@ -687,7 +677,7 @@ static int channel_do_operation(lua_State *L, channel_method_t op)
 	if (!udc->channel) {
 		ERROR("Called GET/PUT channel operation on a closed channel.");
 		lua_pushnil(L);
-		lua_pushnumber(L, SERVER_EINIT);
+		lua_pushinteger(L, SERVER_EINIT);
 		lua_newtable(L);
 		return 3;
 	}
@@ -727,51 +717,47 @@ static int channel_do_operation(lua_State *L, channel_method_t op)
 
 	/* Assemble result for passing back to the Lua realm. */
 	push_result(L, result);
-	lua_pushnumber(L, result);
+	lua_pushinteger(L, result);
 	lua_newtable(L);
-	if (op == CHANNEL_GET || channel_data.method == CHANNEL_POST ||
-	    channel_data.method == CHANNEL_PATCH) {
-		push_to_table(L, "http_response_code", channel_data.http_response_code);
-		push_to_table(L, "format",             channel_data.format);
-		#ifdef CONFIG_JSON
-		if (channel_data.format == CHANNEL_PARSE_JSON) {
-			lua_pushstring(L, "json_reply");
-			if (!channel_data.json_reply ||
-			    !json_to_table(L, channel_data.json_reply)) {
-				lua_pushnil(L);
-			}
-			lua_settable(L, -3);
-
-			if (channel_data.json_reply &&
-			    json_object_put(channel_data.json_reply) != 1) {
-				ERROR("JSON object should be freed but was not.");
-			}
-		}
-		#endif
-		if (channel_data.format == CHANNEL_PARSE_RAW) {
-			lua_pushstring(L, "raw_reply");
-			if (!channel_data.raw_reply) {
-				lua_pushnil(L);
-			} else {
-				lua_pushstring(L, channel_data.raw_reply);
-				free(channel_data.raw_reply);
-			}
-			lua_settable(L, -3);
+	push_to_table(L, "http_response_code", channel_data.http_response_code);
+	push_to_table(L, "format",             channel_data.format);
+	#ifdef CONFIG_JSON
+	if (channel_data.format == CHANNEL_PARSE_JSON) {
+		lua_pushstring(L, "json_reply");
+		if (!channel_data.json_reply ||
+			!json_to_table(L, channel_data.json_reply)) {
+			lua_pushnil(L);
 		}
+		lua_settable(L, -3);
 
-		lua_pushstring(L, "received_headers");
-		lua_newtable(L);
-		if (!LIST_EMPTY(channel_data.received_headers)) {
-			struct dict_entry *entry;
-			LIST_FOREACH(entry, channel_data.received_headers, next) {
-				lua_pushstring(L, dict_entry_get_key(entry));
-				lua_pushstring(L, dict_entry_get_value(entry));
-				lua_settable(L, -3);
-			}
+		if (channel_data.json_reply &&
+			json_object_put(channel_data.json_reply) != 1) {
+			ERROR("JSON object should be freed but was not.");
+		}
+	}
+	#endif
+	if (channel_data.format == CHANNEL_PARSE_RAW) {
+		lua_pushstring(L, "raw_reply");
+		if (!channel_data.raw_reply) {
+			lua_pushnil(L);
+		} else {
+			lua_pushstring(L, channel_data.raw_reply);
+			free(channel_data.raw_reply);
 		}
 		lua_settable(L, -3);
 	}
 
+	lua_pushstring(L, "received_headers");
+	lua_newtable(L);
+	if (!LIST_EMPTY(channel_data.received_headers)) {
+		struct dict_entry *entry;
+		LIST_FOREACH(entry, channel_data.received_headers, next) {
+			lua_pushstring(L, dict_entry_get_key(entry));
+			lua_pushstring(L, dict_entry_get_value(entry));
+			lua_settable(L, -3);
+		}
+	}
+	lua_settable(L, -3);
 	dict_drop_db(&header_send);
 	dict_drop_db(&header_receive);
 
@@ -1162,7 +1148,7 @@ static void do_install(lua_State *L, int fdout)
 		result = iresult == FAILURE ? SERVER_EERR : SERVER_OK;
 	}
 	push_result(L, result);
-	lua_pushnumber(L, result);
+	lua_pushinteger(L, result);
 	lua_newtable(L);
 	if (ipc_journal) {
 		const char **iter = ipc_journal;
@@ -1178,7 +1164,7 @@ static void do_install(lua_State *L, int fdout)
 	goto done;
 error:
 	lua_pushnil(L);
-	lua_pushnumber(L, SERVER_EINIT);
+	lua_pushinteger(L, SERVER_EINIT);
 	lua_newtable(L);
 done:
 	if (callback_data.progress_msgq_lock) {
@@ -1235,7 +1221,7 @@ static int lua_suricatta_download(lua_State *L)
 		ERROR("Cannot open %s for writing.", luaL_checkstring(L, -1));
 		lua_pop(L, 2);
 		lua_pushnil(L);
-		lua_pushnumber(L, SERVER_EINIT);
+		lua_pushinteger(L, SERVER_EINIT);
 		lua_newtable(L);
 		return 3;
 	}
@@ -1472,20 +1458,13 @@ static int lua_bootloader_env_apply(lua_State *L)
 /**
  * @brief Get update state from persistent storage (bootloader).
  *
- * @return [Lua] True, or, in case of error, nil.
- *         [Lua] One of pstate's enum values.
+ * @return [Lua] One of pstate's enum values.
  */
 static int lua_pstate_get(lua_State *L)
 {
 	update_state_t state = get_state();
-	if (is_valid_state(state)) {
-		lua_pushboolean(L, true);
-		lua_pushnumber(L, (int)state);
-	} else {
-		lua_pushnil(L);
-		lua_pushnil(L);
-	}
-	return 2;
+	lua_pushinteger(L, is_valid_state(state) ? (int)state : STATE_ERROR);
+	return 1;
 }
 
 
@@ -1585,7 +1564,31 @@ static int suricatta_lua_module(lua_State *L)
 	lua_newtable(L);
 	luaL_setfuncs(L, lua_funcs_bootloader_env, 0);
 	lua_settable(L, -3);
+	lua_settable(L, -3);
 
+	lua_pushstring(L, "ipc");
+	lua_newtable(L);
+	lua_pushstring(L, "sourcetype");
+	lua_newtable(L);
+	push_to_table(L, "SOURCE_UNKNOWN", SOURCE_UNKNOWN);
+	push_to_table(L, "SOURCE_WEBSERVER", SOURCE_WEBSERVER);
+	push_to_table(L, "SOURCE_SURICATTA", SOURCE_SURICATTA);
+	push_to_table(L, "SOURCE_DOWNLOADER", SOURCE_DOWNLOADER);
+	push_to_table(L, "SOURCE_LOCAL", SOURCE_LOCAL);
+	push_to_table(L, "SOURCE_CHUNKS_DOWNLOADER", SOURCE_CHUNKS_DOWNLOADER);
+	lua_settable(L, -3);
+	lua_pushstring(L, "RECOVERY_STATUS");
+	lua_newtable(L);
+	push_to_table(L, "IDLE", IDLE);
+	push_to_table(L, "START", START);
+	push_to_table(L, "RUN", RUN);
+	push_to_table(L, "SUCCESS", SUCCESS);
+	push_to_table(L, "FAILURE", FAILURE);
+	push_to_table(L, "DOWNLOAD", DOWNLOAD);
+	push_to_table(L, "DONE", SUBPROCESS);
+	push_to_table(L, "SUBPROCESS", SUBPROCESS);
+	push_to_table(L, "PROGRESS", PROGRESS);
+	lua_settable(L, -3);
 	lua_settable(L, -3);
 
 	lua_pushstring(L, "status");
@@ -1806,7 +1809,7 @@ static int config_section_to_table(void *setting, void *data)
 			break;
 		case CONFIG_TYPE_BOOL:
 			push_to_table((lua_State *)data, entry->name,
-				      config_setting_get_bool(entry));
+				      (bool)config_setting_get_bool(entry));
 			break;
 		case CONFIG_TYPE_FLOAT:
 			push_to_table((lua_State *)data, entry->name,
@@ -1827,7 +1830,7 @@ static int config_section_to_table(void *setting, void *data)
  * @param  argv   The array of arguments.
  * @return SERVER_OK, or, in case of errors, SERVER_EINIT or SERVER_EERR.
  */
-server_op_res_t server_start(char *fname, int argc, char *argv[])
+static server_op_res_t server_start(const char *fname, int argc, char *argv[])
 {
 	if (suricatta_lua_create() != SERVER_OK) {
 		suricatta_lua_destroy();
@@ -1873,7 +1876,7 @@ server_op_res_t server_start(char *fname, int argc, char *argv[])
  *
  * @return SERVER_OK or, in case of errors, any other from server_op_res_t.
  */
-server_op_res_t server_stop(void)
+static server_op_res_t server_stop(void)
 {
 	server_op_res_t result = map_lua_result(
 	    call_lua_func(gL, SURICATTA_FUNC_SERVER_STOP, 0));
@@ -1885,7 +1888,7 @@ server_op_res_t server_stop(void)
 /**
  * @brief Print the Suricatta Lua module's help text.
  */
-void server_print_help(void)
+static void server_print_help(void)
 {
 	if (suricatta_lua_create() != SERVER_OK) {
 		fprintf(stderr, "Error loading Suricatta Lua module.\n");
@@ -1908,7 +1911,7 @@ void server_print_help(void)
  *
  * @return Polling interval in seconds.
  */
-unsigned int server_get_polling_interval(void)
+static unsigned int server_get_polling_interval(void)
 {
 	int result = call_lua_func(gL, SURICATTA_FUNC_GET_POLLING_INTERVAL, 0);
 	return result >= 0 ? (unsigned int)result : CHANNEL_DEFAULT_POLLING_INTERVAL;
@@ -1923,9 +1926,9 @@ unsigned int server_get_polling_interval(void)
  *         in suricatta/suricatta.c, the others result in suricatta
  *         sleeping again.
  */
-server_op_res_t server_has_pending_action(int *action_id)
+static server_op_res_t server_has_pending_action(int *action_id)
 {
-	lua_pushnumber(gL, *action_id);
+	lua_pushinteger(gL, *action_id);
 	server_op_res_t result = map_lua_result(
 	    call_lua_func(gL, SURICATTA_FUNC_HAS_PENDING_ACTION, 1));
 	if ((lua_gettop(gL) > 0) && (lua_isnumber(gL, -1))) {
@@ -1945,7 +1948,7 @@ server_op_res_t server_has_pending_action(int *action_id)
  *
  * @return SERVER_OK or, in case of errors, any other from server_op_res_t.
  */
-server_op_res_t server_install_update(void)
+static server_op_res_t server_install_update(void)
 {
 	return map_lua_result(call_lua_func(gL, SURICATTA_FUNC_INSTALL_UPDATE, 0));
 }
@@ -1956,7 +1959,7 @@ server_op_res_t server_install_update(void)
  *
  * @return SERVER_OK or, in case of errors, any other from server_op_res_t.
  */
-server_op_res_t server_send_target_data(void)
+static server_op_res_t server_send_target_data(void)
 {
 	return map_lua_result(call_lua_func(gL, SURICATTA_FUNC_SEND_TARGET_DATA, 0));
 }
@@ -1968,7 +1971,7 @@ server_op_res_t server_send_target_data(void)
  * @param  msg  IPC message.
  * @return SERVER_OK or, in case of errors, any other from server_op_res_t.
  */
-server_op_res_t server_ipc(ipc_message *msg)
+static server_op_res_t server_ipc(ipc_message *msg)
 {
 	lua_newtable(gL);
 	push_to_table(gL, "magic",      msg->magic);
@@ -2014,3 +2017,20 @@ server_op_res_t server_ipc(ipc_message *msg)
 	}
 	return result;
 }
+
+static server_t server = {
+	.has_pending_action = &server_has_pending_action,
+	.install_update = &server_install_update,
+	.send_target_data = &server_send_target_data,
+	.get_polling_interval = &server_get_polling_interval,
+	.start = &server_start,
+	.stop = &server_stop,
+	.ipc = &server_ipc,
+	.help = &server_print_help,
+};
+
+__attribute__((constructor))
+static void register_server_lua(void)
+{
+	register_server("lua", &server);
+}
