@@ -30,9 +30,50 @@ static bool trigger = false;
 static struct option long_options[] = {
     {"enable", no_argument, NULL, 'e'},
     {"disable", no_argument, NULL, 'd'},
+    {"server", required_argument, NULL, 'S'},
     {NULL, 0, NULL, 0}};
 static sem_t suricatta_enable_sema;
 
+typedef struct {
+	const char *name;
+	server_t *funcs;
+} server_entry;
+
+static int servers_count = 0;
+static server_entry *servers = NULL;
+static server_t *server = NULL;
+
+bool register_server(const char *name, server_t *srv)
+{
+	if (!name || !srv) {
+		return false;
+	}
+	server_entry *tmp = realloc(servers,
+			(servers_count + 1) * sizeof(server_entry));
+	if (!tmp) {
+		return false;
+	}
+	tmp[servers_count].name = name;
+	tmp[servers_count].funcs = srv;
+	servers_count++;
+	servers = tmp;
+	return true;
+}
+
+static bool set_server(const char *name)
+{
+	if (!name || !strlen(name)) {
+		return false;
+	}
+	for (unsigned int i = 0; i < servers_count; i++) {
+		if (strcmp(name, servers[i].name) == 0) {
+			server = servers[i].funcs;
+			return true;
+		}
+	}
+	return false;
+}
+
 void suricatta_print_help(void)
 {
 	fprintf(
@@ -40,8 +81,17 @@ void suricatta_print_help(void)
 	    "\tsuricatta arguments (mandatory arguments are marked with '*'):\n"
 	    "\t  -e, --enable      Daemon enabled at startup (default).\n"
 	    "\t  -d, --disable     Daemon disabled at startup.\n"
+	    "\t  -S, --server      Suricatta module to run.\n"
 	    );
-	server.help();
+	if (servers_count == 0) {
+		fprintf(stdout, "\tNo compiled-in suricatta modules!\n");
+		return;
+	}
+	for (unsigned int i = 0; i < servers_count; i++) {
+		fprintf(stdout, "\tOptions for suricatta module '%s':\n",
+			servers[i].name);
+		(servers[i].funcs)->help();
+	}
 }
 
 static server_op_res_t suricatta_enable(ipc_message *msg)
@@ -68,9 +118,9 @@ static server_op_res_t suricatta_enable(ipc_message *msg)
 	else {
 	  /*
 	   * check if polling of server is requested via IPC (trigger)
-	   * This allows to force to check if an update is available
+	   * This allows the client to force to check if an update is available
 	   * on the server. This is useful in case the device is not always
-	   * online, and it just checks for update (and then update should run
+	   * online, and it just checks for updates (and then update should run
 	   * immediately) just when online.
 	   */
 	  json_data = json_get_path_key(
@@ -103,7 +153,7 @@ static server_op_res_t suricatta_ipc(int fd)
 		result = suricatta_enable(&msg);
 		break;
 	default:
-		result = server.ipc(&msg);
+		result = server->ipc(&msg);
 		break;
 	}
 
@@ -120,6 +170,12 @@ static int suricatta_settings(void *elem, void  __attribute__ ((__unused__)) *da
 	get_field(LIBCFG_PARSER, elem, "enable",
 		&enable);
 
+	char cfg_server[128];
+	GET_FIELD_STRING_RESET(LIBCFG_PARSER, elem, "server", cfg_server);
+	if (strlen(cfg_server) && set_server(cfg_server)) {
+		TRACE("Suricatta module '%s' selected by configuration file.", cfg_server);
+	}
+
 	return 0;
 }
 
@@ -227,9 +283,16 @@ int start_suricatta(const char *cfgfname, int argc, char *argv[])
 	optind = 1;
 	opterr = 0;
 
-	while ((choice = getopt_long(argc, argv, "de",
+	while ((choice = getopt_long(argc, argv, "deS:",
 				     long_options, NULL)) != -1) {
 		switch (choice) {
+		case 'S':
+			if (!set_server(optarg)) {
+				ERROR("Suricatta module '%s' not registered.", optarg);
+				exit(EXIT_FAILURE);
+			}
+			TRACE("Suricatta module '%s' selected by command line option.", optarg);
+			break;
 		case 'e':
 			enable = true;
 			break;
@@ -240,6 +303,25 @@ int start_suricatta(const char *cfgfname, int argc, char *argv[])
 			break;
 		}
 	}
+	if (!server) {
+		if (servers_count == 0) {
+			ERROR("No compiled-in suricatta modules!");
+			exit(EXIT_FAILURE);
+		}
+		if (servers_count == 1) {
+			if (!set_server(servers[0].name)) {
+				ERROR("Internal Error: One suricatta module "
+				      "available but not found?!");
+				exit(EXIT_FAILURE);
+			}
+			TRACE("Default suricatta module '%s' selected.", servers[0].name);
+
+		} else {
+			ERROR("Multiple suricatta modules available but none "
+			      "selected. See swupdate --help for options.");
+			exit(EXIT_FAILURE);
+		}
+	}
 
 	if (sem_init(&suricatta_enable_sema, 0, 0)) {
 		ERROR("Initialising suricatta enable semaphore failed");
@@ -254,7 +336,7 @@ int start_suricatta(const char *cfgfname, int argc, char *argv[])
 	/*
 	 * Now start a specific implementation of the server
 	 */
-	if (server.start(cfgfname, argc, serverargv) != SERVER_OK) {
+	if (server->start(cfgfname, argc, serverargv) != SERVER_OK) {
 		exit(EXIT_FAILURE);
 	}
 	free(serverargv);
@@ -263,13 +345,13 @@ int start_suricatta(const char *cfgfname, int argc, char *argv[])
 	while (true) {
 		if (enable || trigger) {
 			trigger = false;
-			switch (server.has_pending_action(&action_id)) {
+			switch (server->has_pending_action(&action_id)) {
 			case SERVER_UPDATE_AVAILABLE:
 				DEBUG("About to process available update.");
-				server.install_update();
+				server->install_update();
 				break;
 			case SERVER_ID_REQUESTED:
-				server.send_target_data();
+				server->send_target_data();
 				trigger = true;
 				break;
 			case SERVER_EINIT:
@@ -281,9 +363,9 @@ int start_suricatta(const char *cfgfname, int argc, char *argv[])
 			}
 		}
 
-		for (int wait_seconds = server.get_polling_interval();
+		for (int wait_seconds = server->get_polling_interval();
 			 wait_seconds > 0;
-			 wait_seconds = min(wait_seconds, (int)server.get_polling_interval())) {
+			 wait_seconds = min(wait_seconds, (int)server->get_polling_interval())) {
 			wait_seconds = suricatta_wait(wait_seconds);
 		}
 
